import utils from '../utils/index';

/**
 *  定义滑动状态
 *  1：表示请求上一屏，通过
 * -1：表示请求上一屏，未通过
 *  2：表示请求下一屏，通过
 * -2：表示请求下一屏，未通过
 */
function getStatus() {
    let fn = (status1, status2) => {
        if (this.slide.movedRate > this.slide.boundRate) {
            return status1;
        } else if (this.slide.movedRate < this.slide.boundRate) {
            return status2;
        }
    };

    if (this.move.distanceX > 0) return fn(1, -1);
    if (this.move.distanceX < 0) return fn(2, -2);
}

/**
 *  获取下一滑屏索引
 */
function getNextIndex() {
    switch (getStatus.call(this)) {
        case 1:
            if (this.slide.loop || !this.ifFirst()) return this.slide.index - 1;
            break;
        case 2:
            if (this.slide.loop || !this.ifLast()) return this.slide.index + 1;
            break;
    }
    return this.slide.index;
}

/**
 * 处理touchstart touchmove touchend事件 ****************************************************************************
 */
function getStartHandler() {
    return e => {
        if (this.slide.running) return;
        this.start.pageX = e.targetTouches[0].pageX;
    }
}

function getMoveHandler() {
    return e => {
        window.clearTimeout(this.autoTimeoutId);

        if (this.slide.running) return;
        this.move.distanceX = e.targetTouches[0].pageX - this.start.pageX;

        if (!this.slide.loop) {
            if (this.ifFirst() && this.move.distanceX > 0  || 
                this.ifLast() && this.move.distanceX < 0) this.move.distanceX = this.move.distanceX / 5;
        }
        else if (this.slide.length === 2) {
            if (this.move.distanceX < 0) {
                utils.setTranslateX(this.elems.lastChild, 0);
            } else {
                utils.setTranslateX(this.elems.firstChild, 0);
            }
        }

        this.move.translateX = this.start.translateX + this.move.distanceX;
        utils.setTranslateX(this.elems.content, this.move.translateX);
    }
}

function getEndHandler() {
    return e => {
        if (this.slide.running || this.move.distanceX === 0) return;
        this.slide.movedRate = Math.abs(this.move.distanceX) / this.slide.width;
        this.slideTo(getNextIndex.call(this), true);
    }
}

export default function() {
    this.elems.content.addEventListener('touchstart', getStartHandler.call(this));
    this.elems.content.addEventListener('touchmove', getMoveHandler.call(this));
    this.elems.content.addEventListener('touchend', getEndHandler.call(this));
}
