import utils from '../utils/index';
import init from './init';

let cssTransitionDuration = utils.getCssProperty('transition-duration');

export default class Slide {
    /**
     *  options Object
     *  @property el {DOMElement} wrap element
     *  @property duration {Number} transition duration (ms)
     *  @property auto {Boolean} auto play 
     *  @property delay {Number} auto play delay time (ms)
     */

    constructor(options = {}) {
        init.call(this, options);
    }

    ifFirst() {
        return this.slide.index === 1;
    }

    ifLast() {
        return this.slide.index === this.slide.length;
    }

    prev() {
        this.slideTo(this.slide.index - 1);
    }

    next() {
        this.slideTo(this.slide.index + 1);
    }

    slideTo(nextIndex, isHandleTouch = false) {
        if (this.slide.running || 
            !this.slide.loop && (nextIndex < 1 || nextIndex > this.slide.length) || 
            !isHandleTouch && nextIndex === this.slide.index) return;

        let diffI = nextIndex - this.slide.index;
        if (!isHandleTouch && Math.abs(diffI) >= 1) {
            if (diffI > 0) {
                utils.setTranslateX(this.elems.lastChild, 0);
            } else {
                utils.setTranslateX(this.elems.firstChild, 0);
            }
        }

        this.slide.running = true;
        if (this.slide.index !== nextIndex) this.slideStart();

        this.slide.nextIndex = nextIndex;
        this.cssAnim();
    }

    slided(prevIndex = this.slide.index) {
        this.slide.running = false;
        this.slide.movedRate = 0;

        this.move.distanceX = 0;
        this.move.translateX = this.start.translateX = this.endTranslateX;

        this.slide.index = this.slide.nextIndex;
        this.elems.content.style[cssTransitionDuration] = '';

        if (this.slide.loop) this.handleLoop();

        if (prevIndex !== this.slide.index) {
            this.slideEnd(this.autoTimeoutId);
        }
        if (this.slide.auto) this.auto();
    }

    handleLoop() {
        let s = this.slide;
        let e = this.elems;

        utils.setTranslateX(e.firstChild, 0);
        utils.setTranslateX(e.lastChild, 0);

        if (s.nextIndex === s.length + 1) {
            s.index = s.nextIndex = 1;
            this.endTranslateX = 0;
            utils.setTranslateX(e.content, 0);

        } else if (s.nextIndex === 0) {
            s.index = s.nextIndex = s.length;
            this.endTranslateX = s.width - s.contentWidth;
            utils.setTranslateX(e.content, this.endTranslateX);
        }

        if (s.index === s.length) {
            utils.setTranslateX(e.firstChild, s.contentWidth);
        } else if (s.index === 1) {
            utils.setTranslateX(e.lastChild, -s.contentWidth);
        }

        this.move.translateX = this.start.translateX = this.endTranslateX;
    }

    auto() {
        window.clearTimeout(this.autoTimeoutId);
        this.autoTimeoutId = window.setTimeout(() => {
            this.next();
        }, (this.slide.interval + this.slide.duration));

        this.handleMouseEvent();
    }

    handleMouseEvent() {
        this.elems.content.addEventListener('mouseover', () => {
            window.clearTimeout(this.autoTimeoutId);
        });
        this.elems.content.addEventListener('mouseout', () => {
            this.auto();
        });
    }

    getEndTranslateX(nextIndex = this.slide.nextIndex, curIndex = this.slide.index) {
        let m = this.move;
        let s = this.slide;

        if (curIndex === nextIndex) {
            return m.translateX - m.distanceX;
        }
        else if (Math.abs(curIndex - nextIndex) === 1) {
            if (nextIndex > curIndex) {
                return m.translateX - (s.width + m.distanceX);
            } else {
                return m.translateX + (s.width - m.distanceX);
            }
        }
        else {
            return m.translateX + s.width * (s.index - nextIndex);
        }
    }

    cssAnim() {
        this.endTranslateX = this.getEndTranslateX();
        this.elems.content.style[cssTransitionDuration] = `${this.slide.duration / 1000}s`;
        utils.setTranslateX(this.elems.content, this.endTranslateX);

        setTimeout(() => this.slided(), this.slide.duration);
    }

    transitionEndHandle() {
        this.elems.content.addEventListener('transitionend', () => {
            this.slided();
        });
    }

    get index() {
        return this.slide.index;
    }
    get length() {
        return this.slide.length;
    }
}
